/**
The MIT License (MIT)
Copyright © 2019 John L. Mooney

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#ifndef _PLAYPEN_GRAMMAR_NODE_
#define _PLAYPEN_GRAMMAR_NODE_
#include <string>
#include <vector>
#include <memory>

namespace playpen
{
    class GrammarTree
    {
      protected:
        long intVal;
        double floatVal;
        std::string stringVal;
        std::vector<GrammarTree*> findChildren(GrammarTree* type);
        std::vector<std::unique_ptr<GrammarTree>> leaves;

     public:
        GrammarTree(){}

        long getInt();
        double getFloat();
        void toCode(void);

        virtual ~GrammarTree() = default;

    };

    extern std::unique_ptr<GrammarTree> tree;

    class GrammarTreeInput : public GrammarTree
    {
        public:
        GrammarTreeInput(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeName : public GrammarTree
    {
        public:
        GrammarTreeName(std::string leaf)
        {
            stringVal = leaf;
        }
    };

    class GrammarTreeInteger : public GrammarTree
    {
        public:
        GrammarTreeInteger(long leaf)
        {
            intVal = leaf;
        }
    };

    class GrammarTreeFloat : public GrammarTree
    {
        public:
        GrammarTreeFloat(double leaf)
        {
            floatVal = leaf;
        }
    };

    class GrammarTreeString : public GrammarTree
    {
        public:
        GrammarTreeString(std::string leaf)
        {
            stringVal = leaf;
        }
    };

    class GrammarTreeTypeFloat : public GrammarTree
    {
        public:
        GrammarTreeTypeFloat(){}
    };

    class GrammarTreeTypeInteger : public GrammarTree
    {
        public:
        GrammarTreeTypeInteger(){}
    };

    class GrammarTreeTypeVoid : public GrammarTree
    {
        public:
        GrammarTreeTypeVoid(){}
    };

    class GrammarTreeTypeString : public GrammarTree
    {
        public:
        GrammarTreeTypeString(){}
    };

    class GrammarTreeTypeName: public GrammarTree
    {
        public:
        GrammarTreeTypeName(){}
    };

    class GrammarTreeVariableDeclaration : public GrammarTree
    {
        public:
        GrammarTreeVariableDeclaration(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeVariableDeclarations : public GrammarTree
    {
        public:
        GrammarTreeVariableDeclarations(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeStatement : public GrammarTree
    {
        public:
        GrammarTreeStatement(GrammarTree* leaf)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf));
        }
    };

    class GrammarTreeStatements : public GrammarTree
    {
        public:
        GrammarTreeStatements(){}
        GrammarTreeStatements(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeFunctionPrototype : public GrammarTree
    {
        public:
        GrammarTreeFunctionPrototype(GrammarTree* leaf1, GrammarTree* leaf2, GrammarTree* leaf3)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf3));
        }
    };

    class GrammarTreeFunctionDefinition : public GrammarTree
    {
        public:
        GrammarTreeFunctionDefinition(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeFunctionDefinitions : public GrammarTree
    {
        public:
        GrammarTreeFunctionDefinitions(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeParams : public GrammarTree
    {
        public:
        GrammarTreeParams(GrammarTree* leaf)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf));
        }

        GrammarTreeParams(GrammarTree* leaf1, GrammarTree* leaf2)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf1));
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf2));
        }
    };

    class GrammarTreeBlock : public GrammarTree
    {
        public:
        GrammarTreeBlock(GrammarTree* leaf)
        {
            leaves.push_back(std::unique_ptr<GrammarTree>(leaf));
        }
    };
}
#endif