/**
The MIT License (MIT)
Copyright © 2019 John L. Mooney

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

%code requires {
    #include "../includes/GrammarTree.hpp"
}
%{
    #include <math.h>
    #include <stdio.h>
    #include <stdlib.h>
    #include <memory>
    #include "../includes/GrammarTree.hpp"
    extern std::unique_ptr<playpen::GrammarTree> playpen::tree;
    int yylex (void);
    extern char* yytext;
    extern int   yylineno;
    void yyerror (char const*);
%}

%define api.value.type {playpen::GrammarTree*}

%token COLON SEMICOLON COMMA RIGHTARROW LEFTBRACE RIGHTBRACE EQUAL
LEFTPAREN RIGHTPAREN STRING CHAR INT FLOAT NAME TYPEINT TYPEFLOAT TYPENAME
TYPEVOID TYPESTRING TYPEFUNCTION VOID RETURN


%start input

%%

input:
function_definition function_definitions            { playpen::tree.reset(new playpen::GrammarTreeInput($1, $2)); }

function_prototype:
    TYPEFUNCTION name COLON
    LEFTPAREN params RIGHTPAREN RIGHTARROW vartype            { $$ = new playpen::GrammarTreeFunctionPrototype($2, $5, $8); }

function_definition:
    function_prototype block     { $$ = new playpen::GrammarTreeFunctionDefinition($1, $2); }

function_definitions:
    function_definitions function_definition        { $$ = new playpen::GrammarTreeFunctionDefinitions($1, $2); }
| %empty                            { $$ = nullptr; }

statements:
    statements statement                    { $$ = new playpen::GrammarTreeStatements( $1, $2); }
    | %empty                        { $$ = new playpen::GrammarTreeStatements(); }

statement:
    vardec SEMICOLON                   { $$ = new playpen::GrammarTreeStatement($1); }


block:
    LEFTBRACE statements RIGHTBRACE             { $$ = new playpen::GrammarTreeBlock($2); }

vartype:
    typeint                         { $$ = $1; }
    | typefloat                     { $$ = $1; }
    | typevoid                      { $$ = $1; }
    | typename                      { $$ = $1; }
    | typestring                        { $$ = $1; }

name:
    NAME                            { $$ = new playpen::GrammarTreeName(std::string(yytext)); }

typeint:
    TYPEINT                         { $$ = new playpen::GrammarTreeTypeInteger(); }

typefloat:
    TYPEFLOAT                       { $$ = new playpen::GrammarTreeTypeFloat(); }

typevoid:
    TYPEVOID                        { $$ = new playpen::GrammarTreeTypeVoid(); }

typestring:
    TYPESTRING                      { $$ = new playpen::GrammarTreeTypeString(); }

typename:
    TYPENAME                        { $$ = new playpen::GrammarTreeTypeName(); }

vardec:
    vartype name                        { $$ = new playpen::GrammarTreeVariableDeclaration($1, $2); }

vardecs:
vardec                              { $$ = $1;}
|   vardecs COMMA vardec                    { $$ = new playpen::GrammarTreeVariableDeclarations($1, $2); }

params:
vardecs                             { $$ = $1; }
| typevoid                          { $$ = $1; }

%%

void yyerror(char const* errorCode)
{
    printf("Token: \"%s\" on line: %d\n", yytext, yylineno);
    printf("Error: %s\n",  errorCode);
    //exit(0);
}