    /**
    The MIT License (MIT)
    Copyright © 2021 John L. Mooney

    Permission is hereby granted, free of charge, to any person obtaining a copy of this software
    and associated documentation files (the “Software”), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all copies or substantial
    portions of the Software.

    THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
    TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
    */

%option noyywrap
%{
    #include "../includes/grammar.tab.hpp"

    using namespace std;
%}

float                   (-?[0-9]+\.[0-9]+)
int                     (-?[0-9])[0-9]*
whitespace              [\r\t\v\f]+
linefeed                \n
name                    [a-zA-Z_]+[a-zA-Z0-9_]*
string                  ((["])(?:\\?+.)*(["]))|((['])(?:\\?+.)*([']))

%%
{int}                   { return INT; }
{float}                 { return FLOAT; }
{string}                { return STRING; }
"string"                { return TYPESTRING; }
"integer"               { return TYPEINT; }
"float"                 { return TYPEFLOAT; }
"void"                  { return TYPEVOID; }
"fun"                   { return TYPEFUNCTION; }
"return"                { return RETURN; }
":"                     { return COLON; }
";"                     { return SEMICOLON; }
"{"                     { return LEFTBRACE; }
"}"                     { return RIGHTBRACE; }
"("                     { return LEFTPAREN; }
")"                     { return RIGHTPAREN; }
"->"                    { return RIGHTARROW;}
","                     { return COMMA; }
"="                     { return EQUAL; }
{name}                  { return NAME; }
{linefeed}              ++yylineno;
{whitespace}
%%
